@@ -191,26 +191,14 @@ typedef int vm_fault_t;
  *
  *  struct vm_fault: .virtual_address was removed by:
  *   2016-12-14  1a29d85eb0f19b7d8271923d8917d7b4f5540b3e
- *
- * The containing struct vm_fault was originally called fault_data. It was
- * renamed to vm_fault by this commit:
- *   2007-07-19  d0217ac04ca6591841e5665f518e38064f4e65bd
- *
- * Only define nv_page_fault_va() on kernels that have struct vm_fault.
- * nv_page_fault_va() is only needed on kernels that have struct vm_fault.
- * (Also, none of our drivers attempt to use page faulting on kernels that
- * are old enough to have struct fault_data.)
- *
  */
-#if defined(NV_VM_FAULT_PRESENT)
-    static inline unsigned long nv_page_fault_va(struct vm_fault *vmf)
-    {
-    #if defined(NV_VM_FAULT_HAS_ADDRESS)
-        return vmf->address;
-    #else
-        return (unsigned long)(vmf->virtual_address);
-    #endif
-    }
-#endif // NV_VM_FAULT_PRESENT
+static inline unsigned long nv_page_fault_va(struct vm_fault *vmf)
+{
+#if defined(NV_VM_FAULT_HAS_ADDRESS)
+    return vmf->address;
+#else
+    return (unsigned long)(vmf->virtual_address);
+#endif
+}
 
 #endif // __NV_MM_H__
