@@ -23,6 +23,7 @@
 
 #include "uvm8_channel.h"
 
+#include "uvm8_api.h"
 #include "uvm8_global.h"
 #include "uvm8_hal.h"
 #include "uvm8_procfs.h"
@@ -247,7 +248,7 @@ NV_STATUS uvm_channel_reserve_gpu_to_gpu(uvm_channel_manager_t *channel_manager,
                                          uvm_gpu_t *dst_gpu,
                                          uvm_channel_t **channel_out)
 {
-    const NvU32 dst_gpu_index = uvm_gpu_index(dst_gpu->id);
+    const NvU32 dst_gpu_index = uvm_id_gpu_index(dst_gpu->id);
     NvU32 ce_index = channel_manager->ce_to_use.gpu_to_gpu[dst_gpu_index];
 
     // No recommended CE for the given pair, use default
@@ -719,46 +720,46 @@ static bool ce_usable_for_channel_type(uvm_channel_type_t type, const UvmGpuCopy
 // Returns negative if the first CE should be considered better than the second
 static int compare_ce_for_channel_type(const UvmGpuCopyEngineCaps *ce_caps,
                                        uvm_channel_type_t type,
+                                       NvU32 ce_index0,
                                        NvU32 ce_index1,
-                                       NvU32 ce_index2,
                                        NvU32 *usage_count)
 {
+    const UvmGpuCopyEngineCaps *cap0 = ce_caps + ce_index0;
     const UvmGpuCopyEngineCaps *cap1 = ce_caps + ce_index1;
-    const UvmGpuCopyEngineCaps *cap2 = ce_caps + ce_index2;
 
+    UVM_ASSERT(ce_usable_for_channel_type(type, cap0));
     UVM_ASSERT(ce_usable_for_channel_type(type, cap1));
-    UVM_ASSERT(ce_usable_for_channel_type(type, cap2));
+    UVM_ASSERT(ce_index0 < UVM_COPY_ENGINE_COUNT_MAX);
     UVM_ASSERT(ce_index1 < UVM_COPY_ENGINE_COUNT_MAX);
-    UVM_ASSERT(ce_index2 < UVM_COPY_ENGINE_COUNT_MAX);
-    UVM_ASSERT(ce_index1 != ce_index2);
+    UVM_ASSERT(ce_index0 != ce_index1);
 
     switch (type) {
         case UVM_CHANNEL_TYPE_CPU_TO_GPU:
             // For CPU to GPU fast sysmem read is the most important
-            if (cap1->sysmemRead != cap2->sysmemRead)
-                return cap2->sysmemRead - cap1->sysmemRead;
+            if (cap0->sysmemRead != cap1->sysmemRead)
+                return cap1->sysmemRead - cap0->sysmemRead;
 
             // Prefer not to take up the CEs for nvlink P2P
-            if (cap1->nvlinkP2p != cap2->nvlinkP2p)
-                return cap1->nvlinkP2p - cap2->nvlinkP2p;
+            if (cap0->nvlinkP2p != cap1->nvlinkP2p)
+                return cap0->nvlinkP2p - cap1->nvlinkP2p;
 
             break;
 
         case UVM_CHANNEL_TYPE_GPU_TO_CPU:
             // For GPU to CPU fast sysmem write is the most important
-            if (cap1->sysmemWrite != cap2->sysmemWrite)
-                return cap2->sysmemWrite - cap1->sysmemWrite;
+            if (cap0->sysmemWrite != cap1->sysmemWrite)
+                return cap1->sysmemWrite - cap0->sysmemWrite;
 
             // Prefer not to take up the CEs for nvlink P2P
-            if (cap1->nvlinkP2p != cap2->nvlinkP2p)
-                return cap1->nvlinkP2p - cap2->nvlinkP2p;
+            if (cap0->nvlinkP2p != cap1->nvlinkP2p)
+                return cap0->nvlinkP2p - cap1->nvlinkP2p;
 
             break;
 
         case UVM_CHANNEL_TYPE_GPU_TO_GPU:
             // Prefer the LCE with the most PCEs
             {
-                int pce_diff = (int)hweight32(cap2->cePceMask) - (int)hweight32(cap1->cePceMask);
+                int pce_diff = (int)hweight32(cap1->cePceMask) - (int)hweight32(cap0->cePceMask);
 
                 if (pce_diff != 0)
                     return pce_diff;
@@ -771,15 +772,15 @@ static int compare_ce_for_channel_type(const UvmGpuCopyEngineCaps *ce_caps,
             // for now assume that the number of PCEs is a good measure.
             // TODO: Bug 1735254: Add a direct CE query for local FB bandwidth
             {
-                int pce_diff = (int)hweight32(cap2->cePceMask) - (int)hweight32(cap1->cePceMask);
+                int pce_diff = (int)hweight32(cap1->cePceMask) - (int)hweight32(cap0->cePceMask);
 
                 if (pce_diff != 0)
                     return pce_diff;
             }
 
             // Leave P2P CEs to the GPU_TO_GPU channel type, when possible
-            if (cap1->nvlinkP2p != cap2->nvlinkP2p)
-                return cap1->nvlinkP2p - cap2->nvlinkP2p;
+            if (cap0->nvlinkP2p != cap1->nvlinkP2p)
+                return cap0->nvlinkP2p - cap1->nvlinkP2p;
 
             break;
 
@@ -796,15 +797,15 @@ static int compare_ce_for_channel_type(const UvmGpuCopyEngineCaps *ce_caps,
     }
 
     // By default, prefer less used CEs (within the UVM driver at least)
-    if (usage_count[ce_index1] != usage_count[ce_index2])
-        return usage_count[ce_index1] - usage_count[ce_index2];
+    if (usage_count[ce_index0] != usage_count[ce_index1])
+        return usage_count[ce_index0] - usage_count[ce_index1];
 
     // And CEs that don't share PCEs
-    if (cap1->shared != cap2->shared)
-        return cap1->shared - cap2->shared;
+    if (cap0->shared != cap1->shared)
+        return cap0->shared - cap1->shared;
 
     // Last resort, just order by index
-    return ce_index1 - ce_index2;
+    return ce_index0 - ce_index1;
 }
 
 // Pick default CE for the given channel type, and increment its usage
@@ -893,7 +894,7 @@ void uvm_channel_manager_set_p2p_ce(uvm_channel_manager_t *manager,
                                     uvm_gpu_t *peer,
                                     NvU32 optimal_ce)
 {
-    const NvU32 peer_gpu_index = uvm_gpu_index(peer->id);
+    const NvU32 peer_gpu_index = uvm_id_gpu_index(peer->id);
 
     UVM_ASSERT(manager->gpu != peer);
     UVM_ASSERT(optimal_ce < UVM_COPY_ENGINE_COUNT_MAX);
@@ -958,7 +959,16 @@ static void init_channel_manager_conf(uvm_channel_manager_t *manager)
                 manager->conf.num_gpfifo_entries);
     }
 
-    // 2- Pushbuffer location
+    // 2- Allocation locations
+
+    // Override if the GPU doesn't have memory
+    if (gpu->mem_info.size == 0) {
+        manager->conf.pushbuffer_loc = UVM_BUFFER_LOCATION_SYS;
+        manager->conf.gpfifo_loc     = UVM_BUFFER_LOCATION_SYS;
+        manager->conf.gpput_loc      = UVM_BUFFER_LOCATION_SYS;
+        return;
+    }
+
     manager->conf.pushbuffer_loc = UVM_BUFFER_LOCATION_SYS;
 
     pushbuffer_loc_value = uvm_channel_pushbuffer_loc;
@@ -1149,7 +1159,7 @@ static void channel_print_push_acquires(uvm_push_acquire_info_t *push_acquire_in
         UVM_SEQ_OR_DBG_PRINT(seq,
                              "%s gpu%u:channel%u:%llu",
                              i == 0? " acquiring values" : "",
-                             push_acquire_info->values[i].gpu_id,
+                             uvm_id_value(push_acquire_info->values[i].gpu_id),
                              push_acquire_info->values[i].channel_id,
                              push_acquire_info->values[i].value);
     }
@@ -1246,7 +1256,12 @@ static int nv_procfs_read_manager_pending_pushes(struct seq_file *s, void *v)
     return 0;
 }
 
-UVM_DEFINE_SINGLE_PROCFS_FILE(manager_pending_pushes);
+static int nv_procfs_read_manager_pending_pushes_entry(struct seq_file *s, void *v)
+{
+    UVM_ENTRY_RET(nv_procfs_read_manager_pending_pushes(s, v));
+}
+
+UVM_DEFINE_SINGLE_PROCFS_FILE(manager_pending_pushes_entry);
 
 static NV_STATUS manager_create_procfs(uvm_channel_manager_t *manager)
 {
@@ -1256,7 +1271,10 @@ static NV_STATUS manager_create_procfs(uvm_channel_manager_t *manager)
     if (!uvm_procfs_is_debug_enabled())
         return NV_OK;
 
-    manager->procfs.pending_pushes = NV_CREATE_PROC_FILE("pending_pushes", gpu->procfs.dir, manager_pending_pushes, (void *)manager);
+    manager->procfs.pending_pushes = NV_CREATE_PROC_FILE("pending_pushes",
+                                                         gpu->procfs.dir,
+                                                         manager_pending_pushes_entry,
+                                                         manager);
     if (manager->procfs.pending_pushes == NULL)
         return NV_ERR_OPERATING_SYSTEM;
 
@@ -1270,7 +1288,12 @@ static int nv_procfs_read_channel_info(struct seq_file *s, void *v)
     return 0;
 }
 
-UVM_DEFINE_SINGLE_PROCFS_FILE(channel_info);
+static int nv_procfs_read_channel_info_entry(struct seq_file *s, void *v)
+{
+    UVM_ENTRY_RET(nv_procfs_read_channel_info(s, v));
+}
+
+UVM_DEFINE_SINGLE_PROCFS_FILE(channel_info_entry);
 
 static int nv_procfs_read_channel_pushes(struct seq_file *s, void *v)
 {
@@ -1280,7 +1303,12 @@ static int nv_procfs_read_channel_pushes(struct seq_file *s, void *v)
     return 0;
 }
 
-UVM_DEFINE_SINGLE_PROCFS_FILE(channel_pushes);
+static int nv_procfs_read_channel_pushes_entry(struct seq_file *s, void *v)
+{
+    UVM_ENTRY_RET(nv_procfs_read_channel_pushes(s, v));
+}
+
+UVM_DEFINE_SINGLE_PROCFS_FILE(channel_pushes_entry);
 
 static NV_STATUS channel_create_procfs(uvm_channel_t *channel)
 {
@@ -1297,11 +1325,11 @@ static NV_STATUS channel_create_procfs(uvm_channel_t *channel)
     if (channel->procfs.dir == NULL)
         return NV_ERR_OPERATING_SYSTEM;
 
-    channel->procfs.info = NV_CREATE_PROC_FILE("info", channel->procfs.dir, channel_info, (void *)channel);
+    channel->procfs.info = NV_CREATE_PROC_FILE("info", channel->procfs.dir, channel_info_entry, channel);
     if (channel->procfs.info == NULL)
         return NV_ERR_OPERATING_SYSTEM;
 
-    channel->procfs.pushes = NV_CREATE_PROC_FILE("pushes", channel->procfs.dir, channel_pushes, (void *)channel);
+    channel->procfs.pushes = NV_CREATE_PROC_FILE("pushes", channel->procfs.dir, channel_pushes_entry, channel);
     if (channel->procfs.info == NULL)
         return NV_ERR_OPERATING_SYSTEM;
 
