@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -51,11 +51,11 @@ static NV_STATUS uvm8_register_callbacks(void)
 {
     NV_STATUS status = NV_OK;
 
-    g_exported_uvm8_ops.suspend = uvm_suspend;
-    g_exported_uvm8_ops.resume = uvm_resume;
+    g_exported_uvm8_ops.suspend = uvm_suspend_entry;
+    g_exported_uvm8_ops.resume = uvm_resume_entry;
     g_exported_uvm8_ops.startDevice = NULL;
     g_exported_uvm8_ops.stopDevice  = NULL;
-    g_exported_uvm8_ops.isrTopHalf  = uvm8_isr_top_half;
+    g_exported_uvm8_ops.isrTopHalf  = uvm8_isr_top_half_entry;
 
     // Register the UVM callbacks with the main GPU driver:
     status = uvm_rm_locked_call(nvUvmInterfaceRegisterUvmCallbacks(&g_exported_uvm8_ops));
@@ -111,16 +111,10 @@ NV_STATUS uvm_global_init(void)
     NV_STATUS status;
     UvmPlatformInfo platform_info;
 
-    status = uvm_thread_context_init();
-    if (status != NV_OK) {
-        UVM_ERR_PRINT("uvm_thread_context_init() failed: %s\n", nvstatusToString(status));
-
-        // We enforce thread contexts to be initialized before any locking API
-        // is used. If thread contexts cannot be initialized properly it is not
-        // safe to jump into the teardown path as it involves using locking
-        // mechanisms.
-        return status;
-    }
+    // Initialization of thread contexts happened already, during registration
+    // (addition) of the thread context associated with the UVM module entry
+    // point that is calling this function.
+    UVM_ASSERT(uvm_thread_context_global_initialized());
 
     uvm_mutex_init(&g_uvm_global.global_lock, UVM_LOCK_ORDER_GLOBAL);
     uvm_init_rwsem(&g_uvm_global.pm.lock, UVM_LOCK_ORDER_GLOBAL_PM);
@@ -263,7 +257,7 @@ void uvm_global_exit(void)
     uvm_assert_mutex_unlocked(&g_uvm_global.va_spaces.lock);
     UVM_ASSERT(list_empty(&g_uvm_global.va_spaces.list));
 
-    uvm_thread_context_exit();
+    uvm_thread_context_global_exit();
     uvm_kvmalloc_exit();
 }
 
@@ -278,11 +272,10 @@ static void uvm_gpu_set_isr_suspended(uvm_gpu_t *gpu, bool is_suspended)
     uvm_spin_unlock_irqrestore(&gpu->isr.interrupts_lock);
 }
 
-NV_STATUS uvm_suspend(void)
+static NV_STATUS uvm_suspend(void)
 {
     uvm_va_space_t *va_space = NULL;
-    uvm_thread_context_t *uvm_context;
-    uvm_gpu_id_t gpu_id;
+    uvm_global_gpu_id_t gpu_id;
     uvm_gpu_t *gpu;
 
     // Upon entry into this function, the following is true:
@@ -304,24 +297,19 @@ NV_STATUS uvm_suspend(void)
 
     // Some locks acquired by this function, such as pm.lock, are released
     // by uvm_resume().  This is contrary to the lock tracking code's
-    // expectations, and requires special handling: lock tracking is
-    // disabled in this function, and the thread context's reference count is
-    // adjusted (DEBUG only) for each lock acquisition without a matching
-    // release.
-    uvm_thread_context_disable_lock_tracking();
+    // expectations, so lock tracking is disabled.
+    uvm_thread_context_lock_disable_tracking();
 
     // Take the global power management lock in write mode to lock out
     // most user-facing entry points.
     uvm_down_write(&g_uvm_global.pm.lock);
-    if (UVM_IS_DEBUG())
-        uvm_thread_context_release();
 
     nv_kthread_q_flush(&g_uvm_global.global_q);
 
     // Though global_lock isn't held here, pm.lock indirectly prevents the
     // addition and removal of GPUs, since these operations can currently
     // only occur in response to ioctl() calls.
-    for_each_gpu_id_in_mask(gpu_id, &g_uvm_global.retained_gpus) {
+    for_each_global_gpu_id_in_mask(gpu_id, &g_uvm_global.retained_gpus) {
         gpu = uvm_gpu_get(gpu_id);
 
         // Since fault buffer state may be lost across sleep cycles, UVM must
@@ -355,65 +343,46 @@ NV_STATUS uvm_suspend(void)
     // options, and so aren't suitable for synchronization with pm.lock.
     uvm_mutex_lock(&g_uvm_global.va_spaces.lock);
 
-    list_for_each_entry(va_space, &g_uvm_global.va_spaces.list, list_node) {
+    list_for_each_entry(va_space, &g_uvm_global.va_spaces.list, list_node)
         uvm_va_space_down_write(va_space);
-        if (UVM_IS_DEBUG()) {
-            // uvm_va_space_up_write() acquires three different locks
-            uvm_thread_context_release();
-            uvm_thread_context_release();
-            uvm_thread_context_release();
-        }
-    }
 
     uvm_mutex_unlock(&g_uvm_global.va_spaces.lock);
 
-    if (UVM_IS_DEBUG()) {
-        // Make sure the reference count patching worked as intended
-        uvm_context = uvm_thread_context();
-        UVM_ASSERT(uvm_context && uvm_context->ref_count == 1);
-    }
-
-    uvm_thread_context_enable_lock_tracking();
+    uvm_thread_context_lock_enable_tracking();
 
     g_uvm_global.pm.is_suspended = true;
 
     return NV_OK;
 }
 
-NV_STATUS uvm_resume(void)
+NV_STATUS uvm_suspend_entry(void)
+{
+    UVM_ENTRY_RET(uvm_suspend());
+}
+
+static NV_STATUS uvm_resume(void)
 {
     uvm_va_space_t *va_space = NULL;
-    uvm_thread_context_t *uvm_context;
-    uvm_gpu_id_t gpu_id;
+    uvm_global_gpu_id_t gpu_id;
     uvm_gpu_t *gpu;
 
     g_uvm_global.pm.is_suspended = false;
 
     // Some locks released by this function, such as pm.lock, were acquired
     // by uvm_suspend().  This is contrary to the lock tracking code's
-    // expectations, and requires special handling: lock tracking is
-    // disabled in this function, and the thread context's reference count is
-    // adjusted (DEBUG only) for each lock release without a matching
-    // acquisition.
-    uvm_thread_context_disable_lock_tracking();
+    // expectations, so lock tracking is disabled.
+    uvm_thread_context_lock_disable_tracking();
 
     // Release each VA space's lock.
     uvm_mutex_lock(&g_uvm_global.va_spaces.lock);
 
-    list_for_each_entry(va_space, &g_uvm_global.va_spaces.list, list_node) {
-        if (UVM_IS_DEBUG()) {
-            // uvm_va_space_up_write() releases three different locks
-            uvm_thread_context_retain();
-            uvm_thread_context_retain();
-            uvm_thread_context_retain();
-        }
+    list_for_each_entry(va_space, &g_uvm_global.va_spaces.list, list_node)
         uvm_va_space_up_write(va_space);
-    }
 
     uvm_mutex_unlock(&g_uvm_global.va_spaces.lock);
 
     // pm.lock is held in lieu of global_lock to prevent GPU addition/removal
-    for_each_gpu_id_in_mask(gpu_id, &g_uvm_global.retained_gpus) {
+    for_each_global_gpu_id_in_mask(gpu_id, &g_uvm_global.retained_gpus) {
         gpu = uvm_gpu_get(gpu_id);
 
         // Bring the fault buffer software state back in sync with the
@@ -427,17 +396,9 @@ NV_STATUS uvm_resume(void)
         uvm_gpu_access_counters_set_ignore(gpu, false);
     }
 
-    if (UVM_IS_DEBUG())
-        uvm_thread_context_retain();
     uvm_up_write(&g_uvm_global.pm.lock);
 
-    if (UVM_IS_DEBUG()) {
-        // Make sure the reference count patching worked as intended
-        uvm_context = uvm_thread_context();
-        UVM_ASSERT(uvm_context && uvm_context->ref_count == 1);
-    }
-
-    uvm_thread_context_enable_lock_tracking();
+    uvm_thread_context_lock_enable_tracking();
 
     // Force completion of any release callbacks successfully queued for
     // deferred completion while suspended.  The deferred release
@@ -449,6 +410,11 @@ NV_STATUS uvm_resume(void)
     return NV_OK;
 }
 
+NV_STATUS uvm_resume_entry(void)
+{
+    UVM_ENTRY_RET(uvm_resume());
+}
+
 bool uvm_global_is_suspended(void)
 {
     return g_uvm_global.pm.is_suspended;
@@ -481,9 +447,40 @@ NV_STATUS uvm_global_reset_fatal_error(void)
     return nv_atomic_xchg(&g_uvm_global.fatal_error, NV_OK);
 }
 
-NV_STATUS uvm_api_is_8_supported(UVM_IS_8_SUPPORTED_PARAMS *params, struct file *filp)
+void uvm_global_mask_retain(const uvm_global_processor_mask_t *mask)
 {
-    params->is8Supported = 1;
+    uvm_gpu_t *gpu;
+    for_each_global_gpu_in_mask(gpu, mask)
+        uvm_gpu_retain(gpu);
+}
+
+void uvm_global_mask_release(const uvm_global_processor_mask_t *mask)
+{
+    uvm_global_gpu_id_t gpu_id;
+
+    if (uvm_global_processor_mask_empty(mask))
+        return;
+
+    uvm_mutex_lock(&g_uvm_global.global_lock);
+
+    // Do not use for_each_global_gpu_in_mask as it reads the GPU state and it
+    // might get destroyed
+    for_each_global_gpu_id_in_mask(gpu_id, mask)
+        uvm_gpu_release_locked(uvm_gpu_get(gpu_id));
+
+    uvm_mutex_unlock(&g_uvm_global.global_lock);
+}
+
+NV_STATUS uvm_global_mask_check_ecc_error(uvm_global_processor_mask_t *gpus)
+{
+    uvm_gpu_t *gpu;
+
+    for_each_global_gpu_in_mask(gpu, gpus) {
+        NV_STATUS status = uvm_gpu_check_ecc_error(gpu);
+        if (status != NV_OK)
+            return status;
+    }
+
     return NV_OK;
 }
 
