@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2015 NVIDIA Corporation
+    Copyright (c) 2015-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -185,7 +185,7 @@ static NV_STATUS test_concurrent_pushes(uvm_va_space_t *va_space)
 
     // As noted above, this test does unsafe things that would be detected by
     // lock tracking, opt-out.
-    uvm_thread_context_disable_lock_tracking();
+    uvm_thread_context_lock_disable_tracking();
 
     pushes = uvm_kvmalloc_zero(sizeof(*pushes) * UVM_PUSH_MAX_CONCURRENT_PUSHES);
     if (pushes == NULL) {
@@ -212,7 +212,7 @@ static NV_STATUS test_concurrent_pushes(uvm_va_space_t *va_space)
     }
 
 done:
-    uvm_thread_context_enable_lock_tracking();
+    uvm_thread_context_lock_enable_tracking();
 
     uvm_tracker_deinit(&tracker);
 
@@ -260,9 +260,11 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
     }
     num_non_paused_pushes = channel->num_gpfifo_entries;
 
-    // The UVM driver only allows push interleaving across separate threads, but it is hard to consistenly replicate the
-    // interleaving. Instead, we temporarily disable lock tracking, so we can interleave pushes from a single thread.
-    uvm_thread_context_disable_lock_tracking();
+    // The UVM driver only allows push interleaving across separate threads, but
+    // it is hard to consistenly replicate the interleaving. Instead, we
+    // temporarily disable lock tracking, so we can interleave pushes from a
+    // single thread.
+    uvm_thread_context_lock_disable_tracking();
 
     status = uvm_rm_mem_alloc_and_map_cpu(gpu, UVM_RM_MEM_TYPE_SYS, size, &mem);
     TEST_CHECK_GOTO(status == NV_OK, done);
@@ -338,7 +340,7 @@ static NV_STATUS test_push_interleaving_on_gpu(uvm_gpu_t* gpu)
 
 done:
     uvm_rm_mem_free(mem);
-    uvm_thread_context_enable_lock_tracking();
+    uvm_thread_context_lock_enable_tracking();
 
     return status;
 }
@@ -567,11 +569,11 @@ done:
     return status;
 }
 
-static NV_STATUS test_pushbuffer(void)
+static NV_STATUS test_pushbuffer(uvm_va_space_t *va_space)
 {
     uvm_gpu_t *gpu;
 
-    for_each_global_gpu(gpu) {
+    for_each_va_space_gpu(gpu, va_space) {
         TEST_CHECK_RET(test_max_pushes_on_gpu(gpu) == NV_OK);
         TEST_CHECK_RET(test_idle_chunks_on_gpu(gpu) == NV_OK);
     }
@@ -629,11 +631,11 @@ static NV_STATUS test_timestamp_on_gpu(uvm_gpu_t *gpu)
     return NV_OK;
 }
 
-static NV_STATUS test_timestamp(void)
+static NV_STATUS test_timestamp(uvm_va_space_t *va_space)
 {
     uvm_gpu_t *gpu;
 
-    for_each_global_gpu(gpu)
+    for_each_va_space_gpu(gpu, va_space)
         TEST_CHECK_RET(test_timestamp_on_gpu(gpu) == NV_OK);
 
     return NV_OK;
@@ -683,10 +685,10 @@ static NV_STATUS sync_memcopy(uvm_channel_type_t type, uvm_mem_t *dst, uvm_mem_t
 
 static bool can_do_peer_copies(uvm_va_space_t *va_space, uvm_gpu_t *gpu_a, uvm_gpu_t *gpu_b)
 {
-    if (gpu_a == gpu_b || !uvm_processor_mask_test(&va_space->can_copy_from[gpu_a->id], gpu_b->id))
+    if (gpu_a == gpu_b || !uvm_processor_mask_test(&va_space->can_copy_from[uvm_id_value(gpu_a->id)], gpu_b->id))
         return false;
 
-    UVM_ASSERT(uvm_processor_mask_test(&va_space->can_copy_from[gpu_b->id], gpu_a->id));
+    UVM_ASSERT(uvm_processor_mask_test(&va_space->can_copy_from[uvm_id_value(gpu_b->id)], gpu_a->id));
 
     // TODO: Bug 2028875. Indirect peers are not supported for now.
     if (uvm_gpus_are_indirect_peers(gpu_a, gpu_b))
@@ -702,7 +704,7 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
     NvU32 i;
     NV_STATUS status;
     uvm_gpu_t *gpu, *gpu_a, *gpu_b;
-    uvm_mem_t *mem[UVM_MAX_PROCESSORS] = {NULL};
+    uvm_mem_t *mem[UVM_ID_MAX_PROCESSORS] = {NULL};
     NvU32 *host_ptr;
     const size_t size = 1024 * 1024;
     bool waive = true;
@@ -720,23 +722,23 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
         return NV_OK;
 
     // Alloc and initialize host buffer
-    status = uvm_mem_alloc_sysmem_and_map_cpu_kernel(size, mem + UVM_CPU_ID);
+    status = uvm_mem_alloc_sysmem_and_map_cpu_kernel(size, &mem[UVM_ID_CPU_VALUE]);
     TEST_CHECK_GOTO(status == NV_OK, done);
 
-    host_ptr = (NvU32 *)uvm_mem_get_cpu_addr_kernel(mem[UVM_CPU_ID]);
+    host_ptr = (NvU32 *)uvm_mem_get_cpu_addr_kernel(mem[UVM_ID_CPU_VALUE]);
 
     for (i = 0; i < size / sizeof(NvU32); ++i)
         host_ptr[i] = i + 1;
 
     // Allocate vidmem on each GPU, and map the host buffer
     for_each_va_space_gpu(gpu, va_space) {
-        status = uvm_mem_alloc_vidmem(size, gpu, mem + gpu->id);
+        status = uvm_mem_alloc_vidmem(size, gpu, &mem[uvm_id_value(gpu->id)]);
         TEST_CHECK_GOTO(status == NV_OK, done);
 
-        status = uvm_mem_map_gpu_kernel(mem[gpu->id], gpu);
+        status = uvm_mem_map_gpu_kernel(mem[uvm_id_value(gpu->id)], gpu);
         TEST_CHECK_GOTO(status == NV_OK, done);
 
-        status = uvm_mem_map_gpu_kernel(mem[UVM_CPU_ID], gpu);
+        status = uvm_mem_map_gpu_kernel(mem[UVM_ID_CPU_VALUE], gpu);
         TEST_CHECK_GOTO(status == NV_OK, done);
     }
 
@@ -747,17 +749,23 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
                 continue;
 
             // Copy from CPU to the first GPU, and then zero out the host copy
-            status = sync_memcopy(UVM_CHANNEL_TYPE_CPU_TO_GPU, mem[gpu_a->id], mem[UVM_CPU_ID]);
+            status = sync_memcopy(UVM_CHANNEL_TYPE_CPU_TO_GPU,
+                                  mem[uvm_id_value(gpu_a->id)],
+                                  mem[UVM_ID_CPU_VALUE]);
             TEST_CHECK_GOTO(status == NV_OK, done);
 
             memset(host_ptr, 0, size / sizeof(NvU32));
 
             // Copy from the first GPU to the second GPU
-            status = sync_memcopy(UVM_CHANNEL_TYPE_GPU_TO_GPU, mem[gpu_b->id], mem[gpu_a->id]);
+            status = sync_memcopy(UVM_CHANNEL_TYPE_GPU_TO_GPU,
+                                  mem[uvm_id_value(gpu_b->id)],
+                                  mem[uvm_id_value(gpu_a->id)]);
             TEST_CHECK_GOTO(status == NV_OK, done);
 
             // Copy from the second GPU back to the host, and check result
-            status = sync_memcopy(UVM_CHANNEL_TYPE_GPU_TO_CPU, mem[UVM_CPU_ID], mem[gpu_b->id]);
+            status = sync_memcopy(UVM_CHANNEL_TYPE_GPU_TO_CPU,
+                                  mem[UVM_ID_CPU_VALUE],
+                                  mem[uvm_id_value(gpu_b->id)]);
             TEST_CHECK_GOTO(status == NV_OK, done);
 
             for (i = 0; i < size / sizeof(NvU32); ++i) {
@@ -777,9 +785,9 @@ static NV_STATUS test_push_gpu_to_gpu(uvm_va_space_t *va_space)
 
  done:
     for_each_va_space_gpu(gpu, va_space)
-        uvm_mem_free(mem[gpu->id]);
+        uvm_mem_free(mem[uvm_id_value(gpu->id)]);
 
-    uvm_mem_free(mem[UVM_CPU_ID]);
+    uvm_mem_free(mem[UVM_ID_CPU_VALUE]);
 
     return status;
 }
@@ -814,11 +822,11 @@ NV_STATUS uvm8_test_push_sanity(UVM_TEST_PUSH_SANITY_PARAMS *params, struct file
     if (status != NV_OK)
         goto done;
 
-    status = test_pushbuffer();
+    status = test_pushbuffer(va_space);
     if (status != NV_OK)
         goto done;
 
-    status = test_timestamp();
+    status = test_timestamp(va_space);
     if (status != NV_OK)
         goto done;
 
