@@ -28,6 +28,7 @@
 #endif
 #include "uvmtypes.h"
 #include "uvm_ioctl.h"
+#include "nv_uvm_types.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -997,6 +998,9 @@ typedef enum
     UVM_TEST_PAGE_THRASHING_POLICY_MAX
 } UVM_TEST_PAGE_THRASHING_POLICY;
 
+// This ioctl returns the thrashing mitigation parameters on the current VA
+// space. Note that these values may change after a simulated/emulated GPU is
+// registered on the VA space.
 #define UVM_TEST_GET_PAGE_THRASHING_POLICY              UVM8_TEST_IOCTL_BASE(62)
 typedef struct
 {
@@ -1153,6 +1157,95 @@ typedef struct
     NV_STATUS                       rmStatus;                                           // Out
 } UVM_TEST_DRAIN_REPLAYABLE_FAULTS_PARAMS;
 
+// Get module config PMA batch size in bytes
+#define UVM_TEST_PMA_GET_BATCH_SIZE                     UVM8_TEST_IOCTL_BASE(75)
+typedef struct
+{
+    NvProcessorUuid                 gpu_uuid;                                           // In
+    NvU64                           pma_batch_size;     NV_ALIGN_BYTES(8)               // Out
+
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_PMA_GET_BATCH_SIZE_PARAMS;
+
+// Request PMA's global statistics
+#define UVM_TEST_PMM_QUERY_PMA_STATS                    UVM8_TEST_IOCTL_BASE(76)
+typedef struct
+{
+    NvProcessorUuid                 gpu_uuid;                                           // In
+    UvmPmaStatistics                pma_stats;                                          // Out
+
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_PMM_QUERY_PMA_STATS_PARAMS;
+
+#define UVM_TEST_NUMA_GET_CLOSEST_CPU_NODE_TO_GPU       UVM8_TEST_IOCTL_BASE(77)
+typedef struct
+{
+    NvProcessorUuid                 gpu_uuid;                                           // In
+
+    // On kernels with NUMA support, this entry contains the closest CPU NUMA
+    // node to this GPU. Otherwise, the value will be -1.
+    NvS32                           node_id;                                            // Out
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_NUMA_GET_CLOSEST_CPU_NODE_TO_GPU_PARAMS;
+
+// Test whether the bottom halves have run on the correct CPUs based on the
+// NUMA node locality of the GPU.
+//
+// Failure is reported if:
+//   1. The GPU has serviced faults but the mask tracking which CPUs the
+//      bottom half ran on was empty, or
+//   2. The set of CPUs where the bottom half ran is not a subset of the CPUs
+//      attached to the NUMA node.
+//
+// This IOCTL returns NV_OK on success, NV_ERR_INVALID_STATE on failure, or
+// NV_ERR_NOT_SUPPORTED if UVM thread affinity is not supported.
+#define UVM_TEST_NUMA_CHECK_AFFINITY                    UVM8_TEST_IOCTL_BASE(78)
+typedef struct
+{
+    NvProcessorUuid                 gpu_uuid;                                           // In
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_NUMA_CHECK_AFFINITY_PARAMS;
+
+#define UVM_TEST_VA_SPACE_ADD_DUMMY_THREAD_CONTEXTS     UVM8_TEST_IOCTL_BASE(79)
+typedef struct
+{
+    // Number of thread contexts to add per thread context table entry
+    NvU32                           num_dummy_thread_contexts;                          // In
+
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_VA_SPACE_ADD_DUMMY_THREAD_CONTEXTS_PARAMS;
+
+#define UVM_TEST_VA_SPACE_REMOVE_DUMMY_THREAD_CONTEXTS  UVM8_TEST_IOCTL_BASE(80)
+typedef struct
+{
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_VA_SPACE_REMOVE_DUMMY_THREAD_CONTEXTS_PARAMS;
+
+#define UVM_TEST_THREAD_CONTEXT_SANITY                  UVM8_TEST_IOCTL_BASE(81)
+typedef struct
+{
+    // Iterations to run.
+    NvU32                           iterations;                                         // In
+
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_THREAD_CONTEXT_SANITY_PARAMS;
+
+#define UVM_TEST_THREAD_CONTEXT_PERF                    UVM8_TEST_IOCTL_BASE(82)
+typedef struct
+{
+    // Iterations to run.
+    NvU32                           iterations;                                         // In
+
+    // Delay, in microseconds, between thread context addition and removal
+    NvU32                           delay_us;                                           // In
+
+    // Median time, in nanoseconds, spent in adding and then deleting a thread
+    // context.
+    NvU64                           ns NV_ALIGN_BYTES(8);                               // Out
+
+    NV_STATUS                       rmStatus;                                           // Out
+} UVM_TEST_THREAD_CONTEXT_PERF_PARAMS;
+
 #ifdef __cplusplus
 }
 #endif
