@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2018 NVIDIA Corporation
+    Copyright (c) 2018-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -402,7 +402,7 @@ void uvm_va_space_mm_shutdown(uvm_va_space_mm_t *va_space_mm)
     uvm_va_space_t *va_space = va_space_mm->va_space;
     uvm_gpu_va_space_t *gpu_va_space;
     uvm_gpu_t *gpu;
-    uvm_processor_mask_t gpus_to_flush;
+    uvm_global_processor_mask_t gpus_to_flush;
     LIST_HEAD(deferred_free_list);
 
     // We need to shut down the VA space on mm teardown, but not on ioctls. We
@@ -452,18 +452,18 @@ void uvm_va_space_mm_shutdown(uvm_va_space_mm_t *va_space_mm)
     // Detach all channels to prevent pending untranslated faults to get to this
     // VA space. This also removes those channels from the VA space and puts
     // them on the deferred free list, so only one thread will do this.
-    uvm_processor_mask_zero(&gpus_to_flush);
     uvm_va_space_down_write(va_space);
     uvm_va_space_detach_all_user_channels(va_space, &deferred_free_list);
-    uvm_processor_mask_copy(&gpus_to_flush, &va_space->registered_gpus);
-    uvm_gpu_retain_mask(&gpus_to_flush);
+    uvm_va_space_global_gpus(va_space, &gpus_to_flush);
+    uvm_global_mask_retain(&gpus_to_flush);
     uvm_va_space_up_write(va_space);
 
     // Flush the fault buffer on all GPUs. This will avoid spurious cancels
     // of stale pending translated faults after we set mm_is_dead later.
-    for_each_gpu_in_mask(gpu, &gpus_to_flush)
+    for_each_global_gpu_in_mask(gpu, &gpus_to_flush)
         uvm_gpu_fault_buffer_flush(gpu);
-    uvm_gpu_release_mask(&gpus_to_flush);
+
+    uvm_global_mask_release(&gpus_to_flush);
 
     // Call nvUvmInterfaceUnsetPageDirectory. This has no effect on non-MPS.
     // Under MPS this guarantees that no new GPU accesses will be made using
