@@ -21,7 +21,6 @@
 #include "nv-reg.h"
 #include "conftest/patches.h"
 #include "rmil.h"
-#include "nv-instance.h"
 #include "nv-ibmnpu.h"
 
 #define NV_DEFINE_SINGLE_NVRM_PROCFS_FILE(name) \
@@ -856,17 +855,17 @@ nv_procfs_close_unbind_lock(
 
             if (nv->flags & NV_FLAG_UNBIND_LOCK)
             {
-                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock acquired\n");
+                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock acquired");
             }
             else
             {
-                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "Could not acquire UnbindLock\n");
+                NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "Could not acquire UnbindLock");
             }
         }
         else if ((value == 0) && (nv->flags & NV_FLAG_UNBIND_LOCK))
         {
             nv->flags &= ~NV_FLAG_UNBIND_LOCK;
-            NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock released\n");
+            NV_DEV_PRINTF(NV_DBG_ERRORS, nv, "UnbindLock released");
         }
         up(&nvl->ldata_lock);
     }
@@ -1293,12 +1292,7 @@ int nv_register_procfs(void)
     char nv_dir_name[20];
     struct proc_dir_entry *entry;
 
-    if (nv_multiple_kernel_modules)
-        snprintf(nv_dir_name, sizeof(nv_dir_name),
-                 "driver/nvidia/%d", nv_module_instance);
-    else
-        snprintf(nv_dir_name, sizeof(nv_dir_name),
-                 "driver/%s", nv_device_name);
+    snprintf(nv_dir_name, sizeof(nv_dir_name), "driver/%s", nv_device_name);
 
     nv_dir_name[sizeof(nv_dir_name) - 1] = '\0';
 
